/* Jentchina -CnlServerFiltersTree
 * STRASSER Michel - 20040317		init
 * */

#include "JnaCnlServerFiltersTree.h"
#include "../XmlReaders/pack_XmlReaders.h"
#include <glib.h>
#include <glib-object.h>

static GObjectClass *jnaCnlServerFiltersTree_parentClass;

static void
impl_JnaCnlServerFiltersTree_finalize(GObject *object)
{
	int i;
	JnaCnlServerFiltersTree *self=(JnaCnlServerFiltersTree*)object;

	g_object_unref(self->server);
	
	for(i=1 ; i<self->alreadyUsed ; i++) {
		if(self->array[i].name)
			g_free(self->array[i].name);
	}

	jnaCnlServerFiltersTree_parentClass->finalize(object);
}

static void
JnaCnlServerFiltersTreeClass_init (JnaCnlServerFiltersTreeClass *selfClass)
{
	GObjectClass *objectClass = (GObjectClass *)selfClass;
	jnaCnlServerFiltersTree_parentClass = g_type_class_peek_parent (selfClass);

	objectClass->finalize=impl_JnaCnlServerFiltersTree_finalize;
}

GType		JnaCnlServerFiltersTree_getType		(void)
{
	static GType type = 0;

	if (!type) {
		GTypeInfo info = {
			sizeof (JnaCnlServerFiltersTreeClass),
			NULL, NULL,
			(GClassInitFunc) JnaCnlServerFiltersTreeClass_init,
			NULL, NULL,
			sizeof (JnaCnlServerFiltersTree),
			0,
			NULL, NULL
		};

		type = g_type_register_static (
			G_TYPE_OBJECT,
			"JnaCnlServerFiltersTree", &info, 0);
	}

	return type;
}

static int	
impl_JnaCnlServerFiltersTree_insert		(JnaCnlServerFiltersTree* self,
						 int parent, 	/* 0 if none */
						 int sibling,	/* 0 if none */
						 char* name)	/* node name */
{
	int retval;

	self->alreadyUsed++;
	g_return_val_if_fail(self->alreadyUsed <= self->max, 0);
	retval=self->alreadyUsed;
	
	self->array[retval].name=g_strdup(name);
	self->array[retval].parent=parent;

	if(sibling)
		self->array[sibling].next=retval;
	
	if(parent && !(self->array[parent].child))
		self->array[parent].child=retval;

	return retval;
}
	
static void
impl_JnaCnlServerFiltersTree_fillRecurseArray	(JnaCnlServerFiltersTree* self,
						 GNode* cur,
						 int parent)	
{
	GNode* child;
	int c=0;

	for(child=cur->children ; child ; child=child->next) {
		c=impl_JnaCnlServerFiltersTree_insert(self, parent, c, child->data);
		impl_JnaCnlServerFiltersTree_fillRecurseArray(self, child, c);
	}
}

static void
impl_JnaCnlServerFiltersTree_fillFiltersArray	(JnaCnlServerFiltersTree* self,
						 GObject* xTree)
{
	GNode* cur;
	
	cur=JNA_X_FILTERS_TREE_READER(xTree)->root;

	impl_JnaCnlServerFiltersTree_fillRecurseArray(self, cur, 0);
}


void 
JnaCnlServerFiltersTree_print			(JnaCnlServerFiltersTree* self)
{
	int i;
	JnaCnlFilter* f=self->array;

	g_return_if_fail(JNA_IS_CNL_SERVER_FILTERS_TREE(self));
	
	g_print("Jna::Central::ServerFiltersTree\n");
	g_print(".server='");
	JnaXServerInfoReader_print(JNA_X_SERVER_INFO_READER(self->server));
	g_print(".num='%d' .max='%d' .alreadyUsed='%d'", self->num, self->max, self->alreadyUsed);
	g_print("'.array='\n");
	for(i=0; i <= self->alreadyUsed ; i++) {
		g_print("record '%d' : .name='%s' .parent='%d' .next='%d' .child='%d' .data='%d'\n",
			 	 i, f[i].name, f[i].parent, f[i].next, f[i].child, f[i].data);

	}
	g_print("'\n");
}

int
JnaCnlServerFiltersTree_getData			(JnaCnlServerFiltersTree* self,
						 JnaCnlIter iter)
{
	return 0;
}

void
JnaCnlServerFiltersTree_setData			(JnaCnlServerFiltersTree* self,
						 JnaCnlIter iter,
						 int data)
{
}

void
JnaCnlServerFiltersTree_getServerInfo		(JnaCnlServerFiltersTree* self,
						 char** out_domain,
						 char** out_id,
						 char** out_name)
{
	g_return_if_fail(JNA_IS_CNL_SERVER_FILTERS_TREE(self));
	
	JnaXServerInfoReader_getInfo(JNA_X_SERVER_INFO_READER(self->server), out_domain, out_id, out_name);
}

GObject* 	
JnaCnlServerFiltersTree_new			(GObject* xServer,	/* JnaXServerInfoReader */
						 GObject* xTree,	/* JnaXFiltersTreeReader */
						 int num,
						 GObject* previous)	/* JnaCnlServerFiltersTree */
{
	JnaCnlServerFiltersTree* retval;
	int nNodes;
	static char root[]="/";

	g_return_val_if_fail(JNA_IS_X_SERVER_INFO_READER(xServer), NULL);
	g_return_val_if_fail(JNA_IS_X_FILTERS_TREE_READER(xTree), NULL);
	g_return_val_if_fail(!previous || JNA_IS_CNL_SERVER_FILTERS_TREE(previous), NULL);
	
	retval=g_object_new(JNA_TYPE_CNL_SERVER_FILTERS_TREE, NULL);

	/* BEGIN relatively of the other servers */
	retval->previous=previous;
	retval->next=NULL;
	retval->num=num;
	retval->server=xServer;
	g_object_ref(retval->server);
	/* END relatively of the other servers */

	/* BEGIN contains */
	nNodes=g_node_n_nodes(JNA_X_FILTERS_TREE_READER(xTree)->root, G_TRAVERSE_ALL);
	retval->array=g_new0(JnaCnlFilter, nNodes+1);
	retval->array[0].child=1;
	retval->array[0].name=root;
	retval->max=nNodes+1;
	retval->alreadyUsed=0;
	/* END contains */
	
	impl_JnaCnlServerFiltersTree_fillFiltersArray(retval, xTree);

	return (GObject*) retval;
}


