/* Jentchina::Central::ServerTreeModel
 * STRASSER Michel - 20040405		init
 * STRASSER Michel - 20040412_1102	filling pre-conditions
 * STRASSER Michel - 20040412_1136	adding filters type pre-conditions
 * STRASSER Michel - 20040412_1940	finished to implement (all but paths), not tested
 * */


#include "JnaCnlServerTreeModel.h"
#include "JnaCnlFiltersTree.h"
#include "JnaCnlServerFiltersTree.h"
#include <glib.h>
#include <glib-object.h>
#include "../XmlReaders/pack_XmlReaders.h"
#include <stdlib.h>


static GObjectClass *jnaCnlServerTreeModel_parentClass;

static gint
impl_JnaCnlServerTreeModel_getNColumns		(JentchinaTreeModel* super)
{
	g_return_val_if_fail(JNA_IS_CNL_SERVER_TREE_MODEL(super), 0);

	return 1;
}

static GType
impl_JnaCnlServerTreeModel_getColumnType 	(JentchinaTreeModel* super,
                             			 gint index)
{
	GType retval;
	
	g_return_val_if_fail(JNA_IS_CNL_SERVER_TREE_MODEL(super), G_TYPE_INVALID);
	g_return_val_if_fail(index==0, G_TYPE_INVALID);

	retval=G_TYPE_STRING;

	return retval;
}

static gboolean
impl_JnaCnlServerTreeModel_getIter		(JentchinaTreeModel* super,
						 JentchinaIter* out_iter,
						 char* path)
{
	gboolean retval=TRUE;
	char** splitted;
	JnaCnlServerTreeModel* self=(JnaCnlServerTreeModel*)super;
	int serverN;
	int i, j;
	GObject* serverFiltersTree;
	int curNode;
	
	g_return_val_if_fail(JNA_IS_CNL_SERVER_TREE_MODEL(super), FALSE);
	g_return_val_if_fail(out_iter, FALSE);
	g_return_val_if_fail(path, FALSE);
	
	splitted=g_strsplit(path, ":", 0);
	g_return_val_if_fail(splitted && splitted[0], FALSE);
	
	serverN=atoi(splitted[0]);
	
	g_return_val_if_fail(serverN<JNA_CNL_FILTERS_TREE(self->tree)->nServers, FALSE);

	serverFiltersTree=JNA_CNL_FILTERS_TREE(self->tree)->first;
	for(i=0 ; i<serverN ; i++)
		serverFiltersTree=JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->next;
	
	curNode=0;
	for(i=1 ; splitted[i] ; i++) {
		int n=atoi(splitted[i]);

		curNode=JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->array[curNode].child;
		for(j=0 ; j<n ; j++) {
			curNode=JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->array[curNode].next;	
		}
	}
		
	*out_iter=JnaCnlIter_new(JNA_CNL_FILTERS_TREE(self->tree)->filtersType, serverN, curNode);
	
	g_strfreev(splitted);
	
	return retval;
}

static char* /* newly allocated */
impl_JnaCnlServerTreeModel_getPath		(JentchinaTreeModel* super,
						 JentchinaIter iter)
{
	JnaCnlServerTreeModel* self=(JnaCnlServerTreeModel*)super;
	char* retval;
	int t, s, n, i;
	int curNode;
	GList* l;
	GList* l2;
	GString* gstr;
	GObject* serverFiltersTree;

	g_return_val_if_fail(JNA_IS_CNL_SERVER_TREE_MODEL(super), NULL);
	g_return_val_if_fail(JnaCnlIter_isValid(iter), NULL);
	
	JnaCnlIter_getAllFast(iter, &t, &s, &n);
	g_return_val_if_fail(JNA_CNL_FILTERS_TREE(self->tree)->filtersType==t, NULL);
	g_return_val_if_fail(s<JNA_CNL_FILTERS_TREE(self->tree)->nServers, NULL);
	
	serverFiltersTree=JNA_CNL_FILTERS_TREE(self->tree)->first;
	for(i=0 ; i<s ; i++)
		serverFiltersTree=JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->next;

	l=NULL;
	for(curNode=n ; curNode ; curNode=JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->array[curNode].parent) 
		l=g_list_prepend(l, GINT_TO_POINTER(curNode));		

	l2=l;
	
	gstr=g_string_new("");

	g_string_append_printf(gstr, "%d", s);

	for(l2=l ; l2 ; l2=g_list_next(l2))
		g_string_append_printf(gstr, ":%d", GPOINTER_TO_INT(l2->data));

	retval=gstr->str;
	g_string_free(gstr, FALSE);
	g_list_free(l);

	return retval;
}

static void
impl_JnaCnlServerTreeModel_getValue		(JentchinaTreeModel* super,
						 JentchinaIter iter,
						 gint column,
						 GValue* out_value)
{
	JnaCnlServerTreeModel* self=(JnaCnlServerTreeModel*)super;
	int i;
	int t, s, n;
	GObject* serverFiltersTree;
	char* value;
	
	g_return_if_fail(JNA_IS_CNL_SERVER_TREE_MODEL(self));
	g_return_if_fail(JnaCnlIter_isValid(iter));
	g_return_if_fail(column==0);
	//g_return_if_fail(out_value);
	JnaCnlIter_getAllFast(iter, &t, &s, &n);
	g_return_if_fail(JNA_CNL_FILTERS_TREE(self->tree)->filtersType==t);
	g_return_if_fail(s<JNA_CNL_FILTERS_TREE(self->tree)->nServers);

	serverFiltersTree=JNA_CNL_FILTERS_TREE(self->tree)->first;
	for(i=0 ; i<s ; i++)
		serverFiltersTree=JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->next;

	g_return_if_fail(n<JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->max);


	if(n) { 	/* not server */
		/* g_message("t=%d, s=%d, n=%d", t, s, n); */
		/* JnaCnlServerFiltersTree_print(JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)); */
		value=JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->array[n].name;
	}
	else 	/* server */
		value=JNA_X_SERVER_INFO_READER(JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->server)->name;

	/* g_message("Value : %s", value); */
	g_value_init(out_value, G_TYPE_STRING);
	g_value_set_string(out_value, value);
}

static gboolean
impl_JnaCnlServerTreeModel_iterNext		(JentchinaTreeModel* super,
						 JentchinaIter* inOut_iter)
{
	JnaCnlServerTreeModel* self=(JnaCnlServerTreeModel*)super;
	gboolean retval=TRUE;
	int t, s, n, s2, n2, i;
	GObject* serverFiltersTree;
	
	g_return_val_if_fail(JNA_IS_CNL_SERVER_TREE_MODEL(self), FALSE);
	g_return_val_if_fail(inOut_iter && JnaCnlIter_isValid(*inOut_iter), FALSE);

	JnaCnlIter_getAllFast(*inOut_iter, &t, &s, &n);
	g_return_val_if_fail(JNA_CNL_FILTERS_TREE(self->tree)->filtersType==t, FALSE);

	g_return_val_if_fail(s<JNA_CNL_FILTERS_TREE(self->tree)->nServers, FALSE);
	
	serverFiltersTree=JNA_CNL_FILTERS_TREE(self->tree)->first;

	for(i=0 ; i<s ; i++)
		serverFiltersTree=JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->next;
	g_return_val_if_fail(n<JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->max, FALSE);

	if(n) { /* not a server root */
//#if 0
		n2=JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->array[n].next;
		if(n2)
			*inOut_iter=JnaCnlIter_new(t, s, n2);
		else 
			retval=FALSE;
//#endif
	}
	else {	// server root
		s2=s+1;
		if(s2>=JNA_CNL_FILTERS_TREE(self->tree)->nServers)
			retval=FALSE;
		else
			*inOut_iter=JnaCnlIter_new(t, s2, 0);
	}

	return retval;
}

static gboolean
impl_JnaCnlServerTreeModel_iterChildren		(JentchinaTreeModel* super,
						 JentchinaIter* out_iter,
						 JentchinaIter parent)
{
	JnaCnlServerTreeModel* self=(JnaCnlServerTreeModel*)super;
	gboolean retval=TRUE;

	g_return_val_if_fail(JNA_IS_CNL_SERVER_TREE_MODEL(self), FALSE);
	g_return_val_if_fail(out_iter, FALSE);

	if (parent) { /* not a root */
//#if 0
		int t, s, n, n2, i;
		GObject* serverFiltersTree;
		
		g_return_val_if_fail(JnaCnlIter_isValid(parent), FALSE);
		JnaCnlIter_getAllFast(parent, &t, &s, &n);
		g_return_val_if_fail(JNA_CNL_FILTERS_TREE(self->tree)->filtersType==t, FALSE);

		g_return_val_if_fail(s<JNA_CNL_FILTERS_TREE(self->tree)->nServers, FALSE);
		serverFiltersTree=JNA_CNL_FILTERS_TREE(self->tree)->first;

		for(i=0 ; i<s ; i++)
			serverFiltersTree=JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->next;
		g_return_val_if_fail(n<JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->max, FALSE);

		if(n) { /* not a server root */
				n2=JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->array[n].child;
			if(n2)	/* has child */
				*out_iter=JnaCnlIter_new(t, s, n2);
			else	/* no child */
				return FALSE;
		}
		else {	/* server root */
			if(JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->max) /* server import or export filters */
				*out_iter=JnaCnlIter_new(t, s, 1);
			else	/* server doesn't import/export filters */
				return FALSE;
		}
//#endif
//		return FALSE;
	}
	else {	/* root */
		if(JNA_CNL_FILTERS_TREE(self->tree)->nServers) /* there are already servers */
			*out_iter=JnaCnlIter_new(JNA_CNL_FILTERS_TREE(self->tree)->filtersType, 0, 0);
		else	/* tree is empty */
			return FALSE;
	}

	return retval;
}

static gboolean
impl_JnaCnlServerTreeModel_iterHasChild		(JentchinaTreeModel* super,
						 JentchinaIter iter)
{
	JnaCnlServerTreeModel* self=(JnaCnlServerTreeModel*)super;
	gboolean retval;
	
	g_return_val_if_fail(JNA_IS_CNL_SERVER_TREE_MODEL(self), FALSE);

	if (iter) { /* not a root */
		int t, s, n, n2, i;
		GObject* serverFiltersTree;

		if(!JnaCnlIter_isValid(iter)) 
			g_warning("iter is %d", iter);
		g_return_val_if_fail(JnaCnlIter_isValid(iter), FALSE);
		JnaCnlIter_getAllFast(iter, &t, &s, &n);
		g_return_val_if_fail(JNA_CNL_FILTERS_TREE(self->tree)->filtersType==t, FALSE);

		g_return_val_if_fail(s<JNA_CNL_FILTERS_TREE(self->tree)->nServers, FALSE);

		serverFiltersTree=JNA_CNL_FILTERS_TREE(self->tree)->first;

		for(i=0 ; i<s ; i++)
			serverFiltersTree=JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->next;
		g_return_val_if_fail(n<JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->max, FALSE);

		if(n) { /* not a server root */
//#if 0
			n2=JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->array[n].child;
			if(n2)	/* has child */
				retval=TRUE;
			else	/* no child */
				retval=FALSE;
//#endif
//			return FALSE;
		}
		else {	/* server root */
//#if 0
			if(JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->max) /* server import or export filters */
				retval=TRUE;
			else	/* server doesn't import/export filters */
				retval=FALSE;
//#endif
//			return FALSE;
		}
	}
	else {	/* root */
		if(JNA_CNL_FILTERS_TREE(self->tree)->nServers) /* there are already servers */
			retval=TRUE;
		else	/* tree is empty */
			retval=FALSE;
	}
	
	return retval;
}

static gint
impl_JnaCnlServerTreeModel_iterNChildren	(JentchinaTreeModel* super,
						 JentchinaIter iter)
{
	JnaCnlServerTreeModel* self=(JnaCnlServerTreeModel*)super;
	gint retval=0;
	
	g_return_val_if_fail(JNA_IS_CNL_SERVER_TREE_MODEL(self), 0);
	g_return_val_if_fail(iter, 0);

	if (iter) { /* not a root */
//#if 0
		int t, s, n1, n2, i;
		GObject* serverFiltersTree;
		
		g_return_val_if_fail(JnaCnlIter_isValid(iter), 0);
		JnaCnlIter_getAllFast(iter, &t, &s, &n1);
		g_return_val_if_fail(JNA_CNL_FILTERS_TREE(self->tree)->filtersType==t, 0);

		g_return_val_if_fail(s<JNA_CNL_FILTERS_TREE(self->tree)->nServers, 0);

		serverFiltersTree=JNA_CNL_FILTERS_TREE(self->tree)->first;

		for(i=0 ; i<s ; i++)
			serverFiltersTree=JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->next;
		g_return_val_if_fail(n1<JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->max, 0);

		if(n1) { /* not a server root */
				n2=JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->array[n1].child;
			if(n2) { // Ok, how many
				retval=1;
				
				for(retval=1; n2; retval++) 
					n2=JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->array[n2].next;
			}
			else	/* no child */
				retval=0;
		}
		else {	/* server root */
			if(JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->max) { /* server import or export filters */
				n2=1;
				retval=1;
				
				for(retval=1; n2; retval++) /* how many */
					n2=JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->array[n2].next;
			}
			else	/* server doesn't import/export filters */
				retval=0;
		}
//#endif
//		return 0;
	}
	else 	/* root */
		retval=JNA_CNL_FILTERS_TREE(self->tree)->nServers;

	return retval;
}

static gboolean
impl_JnaCnlServerTreeModel_iterNthChild		(JentchinaTreeModel* super,
						 JentchinaIter* out_iter,
						 JentchinaIter parent,
						 int n)
{
	JnaCnlServerTreeModel* self=(JnaCnlServerTreeModel*)super;
	gboolean retval=TRUE;

	g_return_val_if_fail(JNA_IS_CNL_SERVER_TREE_MODEL(self), FALSE);
	g_return_val_if_fail(out_iter, FALSE);

	if (parent) { /* not a root */
//#if 0
		int t, s, n1, n2, i;
		GObject* serverFiltersTree;
		
		g_return_val_if_fail(JnaCnlIter_isValid(parent), FALSE);
		JnaCnlIter_getAllFast(parent, &t, &s, &n1);
		g_return_val_if_fail(JNA_CNL_FILTERS_TREE(self->tree)->filtersType==t, FALSE);

		g_return_val_if_fail(s<JNA_CNL_FILTERS_TREE(self->tree)->nServers, FALSE);

		serverFiltersTree=JNA_CNL_FILTERS_TREE(self->tree)->first;

		for(i=0 ; i<s ; i++)
			serverFiltersTree=JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->next;
		g_return_val_if_fail(n1<JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->max, FALSE);

		if(n1) { /* not a server root */
					n2=JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->array[n1].child;
			if(n2) { // Ok, we need the Nth
				int j;
				
				for(j=0 ; j<n && n2; j++)
					n2=JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->array[n2].next;

				if(n2) /* if found */
					*out_iter=JnaCnlIter_new(t, s, n2);
				else	/* out of bound */
					return FALSE;
			}
			else	/* no child */
				return FALSE;
		}
		else {	/* server root */
			if(JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->max) { /* server import or export filters */
				int j;
				
				n2=1;
				
				for(j=0 ; j<n && n2; j++)
					n2=JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->array[n2].next;

				if(n2) /* if found */
					*out_iter=JnaCnlIter_new(t, s, n2);
				else	/* out of bound */
					return FALSE;
			}
			else	/* server doesn't import/export filters */
				return FALSE;
		}
//#endif
//		return FALSE;
	}
	else {	/* root */
		if(n<JNA_CNL_FILTERS_TREE(self->tree)->nServers)
			*out_iter=JnaCnlIter_new(JNA_CNL_FILTERS_TREE(self->tree)->filtersType, n, 0);
		else	/* tree is empty */
			return FALSE;
	}

	return retval;
}

static gboolean
impl_JnaCnlServerTreeModel_iterParent		(JentchinaTreeModel* super,
						 JentchinaIter* out_iter,
						 JentchinaIter child)
{
	JnaCnlServerTreeModel* self=(JnaCnlServerTreeModel*)super;
	gboolean retval=TRUE;

	g_return_val_if_fail(JNA_IS_CNL_SERVER_TREE_MODEL(self), FALSE);
	g_return_val_if_fail(out_iter, FALSE);

	if (child) { /* not a root */
		int t, s, n, n2, i;
		GObject* serverFiltersTree;
		
		g_return_val_if_fail(JnaCnlIter_isValid(child), FALSE);
		JnaCnlIter_getAllFast(child, &t, &s, &n);
		g_return_val_if_fail(JNA_CNL_FILTERS_TREE(self->tree)->filtersType==t, FALSE);

		g_return_val_if_fail(s<JNA_CNL_FILTERS_TREE(self->tree)->nServers, FALSE);
		
		serverFiltersTree=JNA_CNL_FILTERS_TREE(self->tree)->first;

		for(i=0 ; i<s ; i++)
			serverFiltersTree=JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->next;
		g_return_val_if_fail(n<JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->max, FALSE); /* invalid */

		if(n) { /* not a server root */
//#if 0
			n2=JNA_CNL_SERVER_FILTERS_TREE(serverFiltersTree)->array[n].parent;
			*out_iter=JnaCnlIter_new(t, s, n2);
//#endif
//			return FALSE;
		}
		else {	/* server root */
			retval=FALSE;
		}
	}
	else 	/* parent of root is root ? */
		retval=FALSE;

	return retval;
}


static void
impl_JnaCnlServerTreeModel_finalize(GObject *object)
{
	JnaCnlServerTreeModel *self=(JnaCnlServerTreeModel*)object;
	
	g_object_unref(self->tree);

	jnaCnlServerTreeModel_parentClass->finalize(object);
}

static void
JnaCnlServerTreeModelClass_init (JnaCnlServerTreeModelClass *selfClass)
{
	GObjectClass *objectClass = (GObjectClass *)selfClass;
	JentchinaTreeModelClass *modelClass= (JentchinaTreeModelClass *)selfClass;
	jnaCnlServerTreeModel_parentClass = g_type_class_peek_parent (selfClass);

	modelClass->getNColumns=impl_JnaCnlServerTreeModel_getNColumns;
	modelClass->getColumnType=impl_JnaCnlServerTreeModel_getColumnType;
	modelClass->getIter=impl_JnaCnlServerTreeModel_getIter;
	modelClass->getPath=impl_JnaCnlServerTreeModel_getPath;
	modelClass->getValue=impl_JnaCnlServerTreeModel_getValue;
	modelClass->iterNext=impl_JnaCnlServerTreeModel_iterNext;
	modelClass->iterChildren=impl_JnaCnlServerTreeModel_iterChildren;
	modelClass->iterHasChild=impl_JnaCnlServerTreeModel_iterHasChild;
	modelClass->iterNChildren=impl_JnaCnlServerTreeModel_iterNChildren;
	modelClass->iterNthChild=impl_JnaCnlServerTreeModel_iterNthChild;
	modelClass->iterParent=impl_JnaCnlServerTreeModel_iterParent;

	objectClass->finalize=impl_JnaCnlServerTreeModel_finalize;
}

GType		JnaCnlServerTreeModel_getType		(void)
{
	static GType type = 0;

	if (!type) {
		GTypeInfo info = {
			sizeof (JnaCnlServerTreeModelClass),
			NULL, NULL,
			(GClassInitFunc) JnaCnlServerTreeModelClass_init,
			NULL, NULL,
			sizeof (JnaCnlServerTreeModel),
			0,
			NULL, NULL
		};

		type = g_type_register_static (
			JENTCHINA_TYPE_TREE_MODEL,
			"JnaCnlServerTreeModel", &info, 0);
	}

	return type;
}

GObject* 	
JnaCnlServerTreeModel_new		(GObject* tree)
{
	JnaCnlServerTreeModel* retval;

	g_return_val_if_fail(JNA_IS_CNL_FILTERS_TREE(tree), NULL);
	
	retval=g_object_new(JNA_TYPE_CNL_SERVER_TREE_MODEL, NULL);
	
	retval->tree=tree;
	g_object_ref(retval->tree);
#define JnaFilterType_EXPORT 1 /* lazy boy */
	if(JNA_CNL_FILTERS_TREE(tree)->filtersType==JnaFilterType_EXPORT) {
		/* JentchinaTreeModel_print(JENTCHINA_TREE_MODEL(retval), -1, -1); */
	}
#undef JnaFilterType_EXPORT
	return (GObject*) retval;
}

