/* Jentchina - Communication sub system
 * STRASSER Michel - 20040131		init
 * STRASSER Michel - 20040225		implementing stubs
 * STRASSER Michel - 20040229		modularisation
 * */


#include "../JnaCom.h"
#include "../../Central/pack_Central.h"
#include "JnaComCorba.h"
#include "../JnaComRendezVous.h"
#include "../JnaComException.h"
#include <glib.h>
#include <glib-object.h>
#include <string.h>
	
static void
impl_JnaComException_checkForException(CORBA_Environment* ev, GObject** out_comException)
{
	switch(etk_exception_type(ev)) {
		case CORBA_NO_EXCEPTION:/* successful outcome*/
			break;
		case CORBA_USER_EXCEPTION:/* a user-defined exception */
#define SIMPLE_EXCEPTION(GE, CE) \
			if (etk_raised_exception_is_a (ev, CE)) { \
				*out_comException=JnaComException_new(GE, ""); \
				break; \
			}
			SIMPLE_EXCEPTION(JNA__COM_EXCEPTION__BadFilterName, ex_Lioubliou_EBadFilterName)
			SIMPLE_EXCEPTION(JNA__COM_EXCEPTION__BadLogin, ex_Lioubliou_EBadLogin)
			SIMPLE_EXCEPTION(JNA__COM_EXCEPTION__InvalidInterfacesStream, ex_Lioubliou_EInvalidInterfacesStream)
			SIMPLE_EXCEPTION(JNA__COM_EXCEPTION__InvalidLsmStream, ex_Lioubliou_EInvalidLsmStream)
			SIMPLE_EXCEPTION(JNA__COM_EXCEPTION__BadDocument, ex_Lioubliou_EBadDocument)
			SIMPLE_EXCEPTION(JNA__COM_EXCEPTION__BadRessourcePath, ex_Lioubliou_EBadRessourcePath)
			SIMPLE_EXCEPTION(JNA__COM_EXCEPTION__BadSessionKey, ex_Lioubliou_EBadSessionKey)
			SIMPLE_EXCEPTION(JNA__COM_EXCEPTION__SessionTimedOut, ex_Lioubliou_ESessionTimedOut)	
#undef SIMPLE_EXCEPTION
			if (etk_raised_exception_is_a (ev, ex_Lioubliou_EFilterError)) {
				Lioubliou_EFilterError *fE=(Lioubliou_EFilterError*)CORBA_exception_value(ev);
				*out_comException=JnaComException_new(JNA__COM_EXCEPTION__FilterError, fE->mesg); 
				break; 
			}
			break;
		case CORBA_SYSTEM_EXCEPTION:
		default: /* standard exception */
			g_critical("System com exception : '%s'",  CORBA_exception_id(ev));
			*out_comException=JnaComException_new(JNA__COM_EXCEPTION__System,  CORBA_exception_id(ev)); 
			// TODO
			break;
	}
}

/* returns a JnaComCorbaRendezVous object*/
GObject*
impl_JnaComCorba_getRendezVous	(JnaCom* selfO,
				 JnaComFilterType filterType,
				 char* filterName,
				 char* user,
				 char* password,
				 GObject* lsmStream,
				 GObject** out_comException)
{
	GObject* retval=NULL; // JnaComRendezVous
	JnaComCorba* self=(JnaComCorba*)selfO;
	Lioubliou_Stream *lsmStreamL;
	Lioubliou_SessionKey sessionKey=NULL;
        CORBA_Environment ev[1];
	Lioubliou_InterfacesList *interfacesList;
	CORBA_short vortexLifeTime;
	GList* interfacesListG=NULL;
	int i;

        CORBA_exception_init(ev);
	lsmStreamL= ORBit_sequence_alloc (TC_CORBA_sequence_CORBA_octet, JNA_CNL_STREAM(lsmStream)->length);
	
	g_memmove(lsmStreamL->_buffer, JNA_CNL_STREAM(lsmStream)->seq, JNA_CNL_STREAM(lsmStream)->length);

	g_message(__PRETTY_FUNCTION__);
	
	sessionKey=Lioubliou_Server_getRendezVous(
			self->service,
			((filterType==JNA_COM_FILTER_TYPE__IMPORT)?Lioubliou_IMPORT:Lioubliou_EXPORT),
			filterName,
			user,
			password,
			lsmStreamL,
			&interfacesList,
			&vortexLifeTime,
			ev);

	impl_JnaComException_checkForException(ev, out_comException);
	g_return_val_if_fail(!(*out_comException), NULL);
	for(i=0; i<interfacesList->_length; i++) {
		interfacesListG=g_list_append(interfacesListG, g_strdup(interfacesList->_buffer[i]));
	}
	
	retval=JnaComRendezVous_new(sessionKey, interfacesListG, vortexLifeTime);
	
	CORBA_free(interfacesList);	
	CORBA_free(lsmStreamL);	
	
	return retval;
}


/* returns a JnaCnlStream, that contains a lsmStream */
GObject*
impl_JnaComCorba_applyFilter	(JnaCom* selfO,
				 char* sessionKey,
				 GObject* interfacesStream,
				 GObject** out_comException)
{
	GObject* retval=NULL; // JnaCnlStream
	JnaComCorba* self=(JnaComCorba*)selfO;
	CORBA_Environment ev[1];
	Lioubliou_Stream* filtered;
	Lioubliou_Stream* ifaces;

        CORBA_exception_init(ev);

	g_print("sessionKey : '%s'", sessionKey);
	
	ifaces= ORBit_sequence_alloc(TC_CORBA_sequence_CORBA_octet, JNA_CNL_STREAM(interfacesStream)->length);
	g_memmove(ifaces->_buffer, JNA_CNL_STREAM(interfacesStream)->seq, JNA_CNL_STREAM(interfacesStream)->length);
	filtered=Lioubliou_Server_applyFilter(self->service, sessionKey, ifaces, ev);

	impl_JnaComException_checkForException(ev, out_comException);
	g_return_val_if_fail(!(*out_comException), NULL);

	retval=JnaCnlStream_new(filtered->_buffer, filtered->_length);

	return retval;
}

/* returns a JnaCnlStream, that contains a PNG picture, an xml stream, etc ...  */
GObject*
impl_JnaComCorba_getRessource	(JnaCom* selfO,
				 char* ressourcePath,
				 GObject** out_comException)
{
	GObject* retval=NULL;
	JnaComCorba* self=(JnaComCorba*)selfO;
	Lioubliou_Stream *ressourceStream;
        CORBA_Environment ev[1];
	
        CORBA_exception_init(ev);
	
	ressourceStream=Lioubliou_Server_getRessource(self->service, ressourcePath, ev);

	impl_JnaComException_checkForException(ev, out_comException);
	g_return_val_if_fail(!(*out_comException), NULL);
       	
	retval=JnaCnlStream_new(ressourceStream->_buffer, ressourceStream->_length);

	CORBA_free(ressourceStream);

	return retval;
}

static void
JnaComCorbaClass_init (JnaComCorbaClass *selfClass)
{
        CORBA_Environment ev[1];
	int argc=1;
	char* argv[2];

        CORBA_exception_init(ev);
	argv[0]=g_strdup("JnaComCorbaClass");
	argv[1]=NULL;
	
	selfClass->orb=CORBA_ORB_init(&argc, argv, "JnaComCorba", ev);
	etk_abort_if_exception(ev, "JnaComCorba - Orb init failed");

	JNA_COM_CLASS(selfClass)->getRendezVous=impl_JnaComCorba_getRendezVous;
	JNA_COM_CLASS(selfClass)->applyFilter=impl_JnaComCorba_applyFilter;
	JNA_COM_CLASS(selfClass)->getRessource=impl_JnaComCorba_getRessource;

	g_free(argv[0]);
}

GType		JnaComCorba_getType		(void)
{
	static GType type = 0;

	if (!type) {
		GTypeInfo info = {
			sizeof (JnaComCorbaClass),
			NULL, NULL,
			(GClassInitFunc) JnaComCorbaClass_init,
			NULL, NULL,
			sizeof (JnaComCorba),
			0,
			NULL, NULL
		};

		type = g_type_register_static (
			JNA_TYPE_COM,
			"JnaComCorba", &info, 0);
	}

	return type;
}

GObject* 	
JnaComCorba_new			(char* options)
{
	JnaComCorba* retval;
        CORBA_Environment ev[1];

	g_return_val_if_fail(options, NULL);
	
        CORBA_exception_init(ev);
	retval=g_object_new(JNA_TYPE_COM_CORBA, NULL);
	retval->service=CORBA_OBJECT_NIL;
	
	retval->service=(Lioubliou_Server)CORBA_ORB_string_to_object(
			JNA_COM_CORBA_CLASS(G_OBJECT_GET_CLASS(retval))->orb, options, ev);

	if (etk_raised_exception (ev)) {
		g_critical ("Can't use service %s (ior was '%s')", CORBA_exception_id (ev), options);
		CORBA_exception_free (ev); 
		retval=NULL;
	}

	return (GObject*) retval;
}

