/* Jentchina - Communication sub system
 * STRASSER Michel - 20040131		init
 * */


#include "JnaComException.h"
#include <glib.h>
#include <glib-object.h>

static GObjectClass *jnaComException_parentClass;

static char exceptionStringTab[][40]={
	"None",
	"BadFilterName",
	"BadLogin",
	"FilterError",
	"InvalidInterfacesStream",
	"InvalidLsmStream",
	"BadDocument",
	"BadRessourcePath",
	"BadSessionKey",
	"SessionTimedOut"
};

static void
impl_JnaComException_finalize(GObject *object)
{
	JnaComException *self=(JnaComException*)object;

	if(self->mesg)
		g_free(self->mesg);
	jnaComException_parentClass->finalize(object);
}

static void
JnaComExceptionClass_init (JnaComExceptionClass *selfClass)
{
	GObjectClass *objectClass = (GObjectClass *)selfClass;
	jnaComException_parentClass = g_type_class_peek_parent (selfClass);

	objectClass->finalize=impl_JnaComException_finalize;
}

GType		JnaComException_getType		(void)
{
	static GType type = 0;

	if (!type) {
		GTypeInfo info = {
			sizeof (JnaComExceptionClass),
			NULL, NULL,
			(GClassInitFunc) JnaComExceptionClass_init,
			NULL, NULL,
			sizeof (JnaComException),
			0,
			NULL, NULL
		};

		type = g_type_register_static (
			G_TYPE_OBJECT,
			"JnaComException", &info, 0);
	}

	return type;
}

GObject* 	
JnaComException_new			(JnaComExceptionType type,	
					 char* mesg)
{
	JnaComException* retval;

	g_return_val_if_fail(type>0 && type < JNA__COM_EXCEPTION__Last, NULL);
	
	retval=g_object_new(JNA_TYPE_COM_EXCEPTION, NULL);
	
	retval->type=type;
	retval->mesg=g_strdup(mesg);

	return (GObject*) retval;
}

char*
JnaComException_toString		(JnaComException* self)
{
	char* retval=NULL;

	g_return_val_if_fail(JNA_IS_COM_EXCEPTION(self), retval);

	if(self->mesg)
		retval=g_strconcat("JnaComException - ", exceptionStringTab[self->type], " - ", self->mesg, NULL);
	else
		retval=g_strconcat("JnaComException - ", exceptionStringTab[self->type], NULL);
	
	return retval;
}

void
JnaComException_print			(JnaComException* self)
{
	char* str;

	g_return_if_fail(JNA_IS_COM_EXCEPTION(self));
	
	str=JnaComException_toString(self);
	g_print(str);
	g_free(str);
}


