/* Jentchina -XServerInfoReader
 * STRASSER Michel - 20040206		init
 * STRASSER Michel - 20040217		adding id and domain
 * */

#include "JnaXServerInfoReader.h"
#include "JnaXCommon.h"
#include <glib.h>
#include <glib-object.h>
#include <libxml/parser.h>
#include <libxml/tree.h>
#include <libxml/xpath.h>
#include <string.h>
#include "../Central/JnaCnlStream.h"

static GObjectClass *jnaXServerInfoReader_parentClass;

static void
impl_JnaXServerInfoReader_finalize(GObject *object)
{
	JnaXServerInfoReader *self=(JnaXServerInfoReader*)object;

	g_free(self->name);
	
	jnaXServerInfoReader_parentClass->finalize(object);
}

static void
JnaXServerInfoReaderClass_init (JnaXServerInfoReaderClass *selfClass)
{
	GObjectClass *objectClass = (GObjectClass *)selfClass;
	jnaXServerInfoReader_parentClass = g_type_class_peek_parent (selfClass);

	objectClass->finalize=impl_JnaXServerInfoReader_finalize;
}

char*
JnaXServerInfoReader_toString		(JnaXServerInfoReader* self)
{
	char* retval;
	
	g_return_val_if_fail(JNA_IS_X_SERVER_INFO_READER(self), NULL);
	retval=g_strdup_printf("Jna::XmlReaders::ServerInfoReader .domain='%s' .id='%s' .name='%s'\n", 
			self->domain,
			self->id,
			self->name);

	return retval;
}

void
JnaXServerInfoReader_print		(JnaXServerInfoReader* self)
{
	char* str;
	
	g_return_if_fail(JNA_IS_X_SERVER_INFO_READER(self));
	
	str=JnaXServerInfoReader_toString(self);
	g_print(str);
	g_free(str);
}

GType		JnaXServerInfoReader_getType		(void)
{
	static GType type = 0;

	if (!type) {
		GTypeInfo info = {
			sizeof (JnaXServerInfoReaderClass),
			NULL, NULL,
			(GClassInitFunc) JnaXServerInfoReaderClass_init,
			NULL, NULL,
			sizeof (JnaXServerInfoReader),
			0,
			NULL, NULL
		};

		type = g_type_register_static (
			G_TYPE_OBJECT,
			"JnaXServerInfoReader", &info, 0);
	}

	return type;
}

static gboolean	/* TRUE if no error */
impl_JnaXServerInfoReader_completeFromXmlDoc	(JnaXServerInfoReader* self,
						 xmlDocPtr doc)
{
	gboolean retval=TRUE;
	xmlXPathContextPtr xpath_context;
	xmlXPathObjectPtr xpath_object=NULL;
	xmlNodePtr node;
	int max;

	xpath_context = xmlXPathNewContext (doc);

	xpath_object = xmlXPathEval ("/TblSi/TblSiName", xpath_context);
	max=xmlXPathNodeSetGetLength(xpath_object->nodesetval);
	g_return_val_if_fail(max>0, FALSE);
	node=xmlXPathNodeSetItem(xpath_object->nodesetval, 0);
	self->name=g_strdup((char *) xmlNodeGetContent(node));
	xmlXPathFreeObject(xpath_object);

	xpath_object = xmlXPathEval ("/TblSi/TblSiDomain", xpath_context);
	max=xmlXPathNodeSetGetLength(xpath_object->nodesetval);
	g_return_val_if_fail(max>0, FALSE);
	node=xmlXPathNodeSetItem(xpath_object->nodesetval, 0);
	self->domain=g_strdup((char *) xmlNodeGetContent(node));
	xmlXPathFreeObject(xpath_object);

	xpath_object = xmlXPathEval ("/TblSi/TblSiId", xpath_context);
	max=xmlXPathNodeSetGetLength(xpath_object->nodesetval);
	g_return_val_if_fail(max>0, FALSE);
	node=xmlXPathNodeSetItem(xpath_object->nodesetval, 0);
	self->id=g_strdup((char *) xmlNodeGetContent(node));
	xmlXPathFreeObject(xpath_object);

	xmlXPathFreeContext(xpath_context);

	return retval;
}

GObject* 	
JnaXServerInfoReader_newFromStream		(GObject* stream)	/* JnaCnlStream */
{
	JnaXServerInfoReader* retval;
	xmlDocPtr doc;

	g_return_val_if_fail(JNA_IS_CNL_STREAM(stream), NULL);

	doc=xmlParseMemory(JNA_CNL_STREAM(stream)->seq, JNA_CNL_STREAM(stream)->length);

	if(!doc) {
		g_critical("Stream %s is not a valid xml stream", JNA_CNL_STREAM(stream)->seq);
		return NULL;
	}

	retval=g_object_new(JNA_TYPE_X_SERVER_INFO_READER, NULL);

	if(!impl_JnaXServerInfoReader_completeFromXmlDoc(retval, doc))
		return NULL;

	xmlFreeDoc(doc);

	return (GObject*) retval;
}

GObject* 	
JnaXServerInfoReader_new			(char* xmlFile)
{
	JnaXServerInfoReader* retval;
	xmlDocPtr doc;

	g_return_val_if_fail(xmlFile, NULL);
	
	doc=xmlParseFile(xmlFile);

	if(!doc) {
		g_critical("File %s is not a valid xml file", xmlFile);
		return NULL;
	}

	retval=g_object_new(JNA_TYPE_X_SERVER_INFO_READER, NULL);

	if(!impl_JnaXServerInfoReader_completeFromXmlDoc(retval, doc))
		return NULL;

	xmlFreeDoc(doc);

	return (GObject*) retval;
}

void
JnaXServerInfoReader_getInfo		(JnaXServerInfoReader* self,
					 char** out_domain, 	/* NOT newly allocated */
					 char** out_id,		/* NOT newly allocated */
					 char** out_name)	/* NOT newly allocated */
{
	*out_domain=self->domain;
	*out_id=self->id;
	*out_name=self->name;
}



